#include <iostream>

#include "kekengine.h"
#include <GL/glew.h>
#include <GLFW/glfw3.h>

using namespace kek;

static ButtonElement *button;
static KeyboardCapture capture = KEK_INVALID_ID;

void periodicCallback(GLFWwindow *window, void *data){

}

void keyCallback(GLFWwindow *window, int key, int scancode, int action, int mods, void *data){

}

void mouseButtonCallback(GLFWwindow *window, int button, int action, int mods, void *data){

}

void onButtonClick(void *data) {
	button->color = Colors::RED;
	capture = Input::captureKeyboardInput(KeyCharCallback([](GLFWwindow *window, unsigned int codepoint, void *data) {
		std::u32string str = Unicode::convertStdToU32(button->text->getText());
		if(codepoint == KEK_INPUT_DELETE) {
			str = str.substr(0, str.length() - 1);
		}else {
			str.push_back(codepoint);
		}
		button->text->setText(Unicode::convertU32ToStd(str));
	}, nullptr), KeyCallback([](GLFWwindow *window, int key, int scancode, int action, int mods, void *data) {
		if(key == GLFW_KEY_ENTER && action == GLFW_PRESS) {
			Input::uncaptureKeyboardInput(capture);
		}
	}, nullptr), Callable([](void *data) {
		button->color = Colors::WHITE;
	}, nullptr));
}

int main(int argc, char **argv) {
	if(Engine::init() != KEK_SUCCESS) return 1;

	MemoryBuffer *buf = Resource::loadResource("object/sphere/Sphere.obj");
	Mesh *mesh = ObjParser::parseMesh(buf, "object/sphere/");
	delete buf;

	GameObject *test = new GameObject();
	test->addMesh(mesh);

	Scene *scene = new Scene();
	scene->addObject(test);

	for(int i = 0; i < 10; i++) {
		GameObject *test2 = new GameObject();
		test2->addMesh(ObjParser::loadMesh("object/cube_colored/Cube.obj"));
		test2->moveTo(glm::vec3(0.0f, 5.0f, 3 * i));
		scene->addObject(test2);
	}

	PointLight *light = new PointLight(glm::vec3(1), 1, 0, 1);
	scene->lights->add(light);

	//DirectionalLight *l = new DirectionalLight(glm::vec3(1), glm::vec3(1, -1, 1));
	//scene->lights->add(l);

	//SpotLight *spot = new SpotLight(glm::vec3(1), glm::vec3(0, 0, -1), 1, 0, 1, glm::radians(8.0f), glm::radians(15.0f));
	//spot->moveTo(glm::vec3(0, 0, 5));
	//scene->lights->add(spot);

	Engine::setActiveScene(scene);

	Input::addPeriodicCallback(PeriodicCallback(periodicCallback, nullptr));
	Input::addKeyListener(KeyCallback(keyCallback, nullptr));
	Input::addMouseButtonListener(MouseButtonCallback(mouseButtonCallback, nullptr));

	button = new ButtonElement(px(10), px(100), px(200), px(50));
	button->text->color = Colors::BLACK;
	button->color = Colors::WHITE;
	button->hoverColor = Colors::GRAY;
	button->text->setText("Hello There!");
	button->onClick = Callable(onButtonClick, nullptr);
	UI::addElement(button);

	if(Engine::start() != KEK_SUCCESS) return 1;
	return 0;
}
